import { EntityRepository, getConnection, Repository } from "typeorm";
import { TestimonialsEntity } from "./entities/testimonials.entity";
import { CreateTestimonialsDto } from "./dto/create-testimonials.dto";
import { UpdateTestimonialsDto } from "./dto/update-testimonials.dto";

@EntityRepository(TestimonialsEntity)
export class TestimonialsRepository extends Repository<TestimonialsEntity> {
    async createEntity(dto: CreateTestimonialsDto): Promise<TestimonialsEntity | Error> {
        try {
            const {
                name,
                review,
                rating,
                image
            } = dto;
            const entity = this.create({
                name,
                review,
                rating,
                image
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<TestimonialsEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateTestimonialsDto): Promise<TestimonialsEntity | Error> {
        try {
            await this.update(id, {
                name: dto.name,
                review: dto.review,
                rating: dto.rating,
                image: dto.image
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<TestimonialsEntity[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(TestimonialsEntity)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}