import {Body, Controller, HttpCode, HttpStatus, Post, Req, UseGuards} from '@nestjs/common';
import {AuthGuard} from 'src/core/guards/auth.guard';
import {Constants} from '../../utils/constants';
import {AuthService} from './auth.service';
import {LoginAuthDto} from './dto/login-auth.dto';
import {RegisterAuthDto} from './dto/register-auth.dto';
import {ResetPasswordRequestDto} from './dto/reset-password.dto';
import { CheckVerifyHashPasswordDto } from './dto/check-verify-hash-password-token';
import { UpdatePasswordUsingVerifyHashPasswordTokenDto } from './dto/update-password-using-verify-hash-password-token';
import { GoogleLoginDto } from './dto/create-google-login.dto';

@Controller({
    path: 'auth',
    version: Constants.API_VERSION_1
})
export class AuthController {

    constructor(private readonly authService: AuthService) {
    }

    @Post('customer/login')
    async login(@Body() dto: LoginAuthDto) {
        return await this.authService.login(dto);
    }

    @Post('admin/login')
    async loginAdmin(@Body() dto: LoginAuthDto) {
        return await this.authService.loginAdmin(dto);
    }

    @Post('/change-password')
    @HttpCode(HttpStatus.OK)
    @UseGuards(AuthGuard)
    async changePassword(@Body('currentPassword') currentPassword: string, @Body('newPassword') newPassword: string, @Req() request: any) {
        return await this.authService.changePassword(currentPassword, newPassword, request);
    }

    @Post('/forgot-password')
    async forgotPassword(@Body() dto: ResetPasswordRequestDto) {
        return await this.authService.forgotPassword(dto);
    }


    @Post('/register')
    async register(@Body() dto: RegisterAuthDto) {
        return await this.authService.register(dto);
    }

    
    @Post('/validate-token')
    async verifyToken(@Req() reg: any) {
        return await this.authService.validateRequest(reg);
    }

    @Post('/verify-hash-token')
    async verifyHashToken(
        @Body() dto: CheckVerifyHashPasswordDto
    ) {
        return await this.authService.verifyHashPasswordTokenRequest(
            dto
        );
    }

    @Post('/hash-token/update-password')
    async updatePasswordUsingHashToken(
        @Body() dto: UpdatePasswordUsingVerifyHashPasswordTokenDto
    ) {
        return await this.authService.updatePasswordUsingTokenRequest(
            dto
        );
    }

    @Post("/google-login")
    doGoogleLogin(@Body() dto: GoogleLoginDto) {
        return this.authService.doGoogleLoginService(dto);
    }
}
