import {EntityRepository, getConnection, Repository} from "typeorm";
import { StripeCheckoutEntity } from "./entity/stripe-checkout.entity";
import { CreateStripeCheckoutDto } from "./dto/create-stripe-checkout.dto";
import { UpdateStripeCheckoutDto } from "./dto/update-stripe-checkout.dto";


@EntityRepository(StripeCheckoutEntity)
export class StripeCheckoutRepository extends Repository<StripeCheckoutEntity> {
    async createEntity(
        dto: CreateStripeCheckoutDto
    ): Promise<StripeCheckoutEntity | Error> {
        try {
            const { checkoutSessionData, checkoutResponseData, sessionID, stripeCheckoutID, paymentStatus, stripeCheckoutStatus, amount_subtotal, amount_total, orderID, products, shippingPackages} = dto;
            const entity = this.create({
                checkoutSessionData,
                checkoutResponseData,
                sessionID,
                stripeCheckoutID,
                paymentStatus,
                stripeCheckoutStatus,
                amount_subtotal,
                amount_total,
                orderID,
                products,
                shippingPackages
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(
        id: string
    ): Promise<StripeCheckoutEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(
        id: string,
        dto: UpdateStripeCheckoutDto
    ): Promise<StripeCheckoutEntity | Error> {
        try {
            await this.update(id, {
                checkoutResponseData: dto.checkoutResponseData,
                paymentStatus: dto.paymentStatus,
                stripeCheckoutStatus: dto.stripeCheckoutStatus,
                amount_subtotal: dto.amount_subtotal,
                amount_total: dto.amount_total,
                orderID: dto.orderID,
                shippingPackages: dto.shippingPackages
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<StripeCheckoutEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async findProductsByStripeCheckoutID(stripeCheckoutID: string): Promise<string[] | Error> {
        try {
            const products = await this.createQueryBuilder("stripeCheckout")
                .select("stripeCheckout.products", "products")
                .where("stripeCheckout.stripeCheckoutID = :stripeCheckoutID", { stripeCheckoutID })
                .andWhere("stripeCheckout.isDeleted = :isDeleted", { isDeleted: false })
                .getRawOne();
    
            if (products) {
                return JSON.parse(products.products);
            }
    
            return [];
        } catch (e) {
            return e;
        }
    }

    async findShippingPackagesByStripeCheckoutID(stripeCheckoutID: string): Promise<string[] | Error> {
        console.log(stripeCheckoutID);
        try {
            const shippingPackages = await this.createQueryBuilder("stripeCheckout")
                .select("stripeCheckout.shippingPackages", "shippingPackages")
                .where("stripeCheckout.stripeCheckoutID = :stripeCheckoutID", { stripeCheckoutID })
                .andWhere("stripeCheckout.isDeleted = :isDeleted", { isDeleted: false })
                .getRawOne();
    
            if (shippingPackages) {
                return JSON.parse(shippingPackages.shippingPackages);
            }
    
            return [];
        } catch (e) {
            return e;
        }
    }
    
    
    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(StripeCheckoutEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}