import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { ColorsRepository } from './color-repository';
import { CreateColorDto } from './dto/create-color.dto';
import { UpdateColorDto } from './dto/update-color.dto';
import { Color } from './entity/color.entity';

@Injectable()
export class ColorService {
    constructor(
        @InjectRepository(ColorsRepository)
        private readonly repository: ColorsRepository
    ) {
    }

    async create(dto: CreateColorDto): Promise<Color | Error> {
        const res: any = await this.repository.findByColorName(dto.colorName);
        if (res.length > 0) {
            throw new HttpException(
                "Color name already exists.",
                HttpStatus.BAD_REQUEST
            );
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(200, "Color created successfully!", "data", data);
    }

    async findAll(): Promise<Color[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(200, "Color Data Fetched", "data", data);
    }

    async update(id: string, dto: UpdateColorDto): Promise<Color | Error> {
        const color: any = await this.repository.findOneEntity(id);
        if (!color) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const data = await this.repository.updateColor(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Color updated successfully!", "data", data);
    }

    async findOne(id: string): Promise<Color[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(200, "Color Data Fetched", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.buildDeletedData(data, 200, "Color deleted successfully!");
    }
}
