import { Between, EntityRepository, FindOperator, getConnection, getManager, Repository } from "typeorm";
import { CreateProductReviewDto } from "./dto/create-product-review.dto";
import { ProductReview } from "./entities/product-review.entity";
import { UpdateProductReviewDto } from "./dto/update-product-review.dto";

@EntityRepository(ProductReview)
export class ProductReviewRepository extends Repository<ProductReview> {
    async createEntity(dto: CreateProductReviewDto): Promise<ProductReview | Error> {
        try {
            const {
                productId,
                productName,
                rating,
                userId,
                userName,
                comment
            } = dto;
            const entity = this.create({
                productId,
                productName,
                rating,
                userId,
                userName,
                comment
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductReview | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findAllEntities(take: number, page: number, query: any): Promise<any | Error> {
        try {
            let strDate: any;
         
            const findOptions: any = {};

            if (query.productName) findOptions.productName = query.productName;
            if (query.rating) findOptions.rating = query.rating;
            if (query.userName) findOptions.userName = query.userName;
            if (query.date) strDate = query.date;

            
            const date = new Date(strDate);
            date.setUTCHours(0, 0, 0, 0);
            
            if (query.date) findOptions.createdAt = Between(
                new Date(date),
                new Date(),
            );
          
            const data = await this.find({
                take, skip: take * (page - 1),
                order: {createdAt: "DESC"},
                where: {
                    ...findOptions,
                    isDeleted: false
                },
            });
            // console.log(data);
            return data;
        } catch (e) {
            return e;
        }
    }
    async findByFilterQuery(query: any): Promise<ProductReview[] | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isDeleted: false
                },
                order: {
                    createdAt: "DESC"
                }
            });
        } catch (e) {
            return e;
        }
    }


    async findAllWithUserInfo(productId: string): Promise<ProductReview[] | Error> {
        try {
            const sql = `SELECT review.*, 
            CONCAT(users.firstName, ' ', users.lastName) as 
            userName, users.image as userImage FROM product_review review
            LEFT JOIN users ON review.userId = users.id 
            WHERE review.productId = '${productId}' AND review.isDeleted = false ;`;
            const entityManager = getManager();
            return entityManager.query(sql);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateProductReviewDto): Promise<ProductReview | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductReview)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}